import 'package:flutter/material.dart';
import 'package:hub/backend/api_requests/api_calls.dart';
import 'package:hub/backend/api_requests/api_manager.dart';
import 'package:hub/components/molecular_components/message_opt_modal/opt_modal_widget.dart';
import 'package:hub/flutter_flow/flutter_flow_model.dart';
import 'package:hub/flutter_flow/flutter_flow_theme.dart';
import 'package:hub/flutter_flow/internationalization.dart';
import 'package:hub/flutter_flow/request_manager.dart';
import 'package:hub/pages/acess_history_page/acess_history_page_widget.dart';
import 'package:hub/shared/helpers/db_helper.dart';
import 'package:hub/shared/utils/dialog_util.dart';
import 'package:hub/shared/utils/log_util.dart';
import 'package:rxdart/rxdart.dart';

class AccessHistoryModel extends FlutterFlowModel<AccessHistoryScreen> {
  // Singleton instance
  static final AccessHistoryModel _instance = AccessHistoryModel._internal();

  // Private constructor
  AccessHistoryModel._internal() : super();

  // Factory constructor to return the singleton instance
  factory AccessHistoryModel() {
    return _instance;
  }

  final DatabaseHelper db = DatabaseHelper();
  late final String devUUID;
  late final String userUUID;
  late final String cliUUID;
  final BehaviorSubject<Map<String, String>> selectedTypeSubject;
  bool isSubjectClosed = false;
  final scaffoldKey = GlobalKey<ScaffoldState>();
  late Function(Function) safeSetState;

  late ScrollController scrollController;
  int pageNumber = 1;
  final int pageSize = 10;
  bool hasData = false;
  bool loading = false;

  String personType = '.*';

  late Future<void> accessFuture;
  List<dynamic> accessWrap = [];

  final unfocusNode = FocusNode();
  final _accessHistoryManager = FutureRequestManager<ApiCallResponse>();
  Future<ApiCallResponse> accessHistory({
    String? uniqueQueryKey,
    bool? overrideCache,
    required Future<ApiCallResponse> Function() requestFn,
  }) =>
      _accessHistoryManager.performRequest(
        uniqueQueryKey: uniqueQueryKey,
        overrideCache: overrideCache,
        requestFn: requestFn,
      );
  void clearAccessHistoryCache() => _accessHistoryManager.clear();
  void clearAccessHistoryCacheKey(String? uniqueKey) =>
      _accessHistoryManager.clearRequest(uniqueKey);

  @override
  void initState(BuildContext context) {
    _initAsync();
  }

  Future<void> _initAsync() async {
    devUUID = await db
        .get(key: 'devUUID', field: 'value')
        .then((value) => value.toString());
    userUUID = await db
        .get(key: 'userUUID', field: 'value')
        .then((value) => value.toString());
    cliUUID = await db
        .get(key: 'cliUUID', field: 'value')
        .then((value) => value.toString());
  }

  void _updateAccessHistoryAction(
      Map<String, String> newType, BuildContext context) {
    if (!isSubjectClosed) {
      final currentType = selectedTypeSubject.value;
      final updatedType = Map<String, String>.from(currentType);
      bool needsUpdate = false;
      newType.forEach((key, newValue) {
        if (currentType[key] != newValue) {
          updatedType[key] = newValue;
          needsUpdate = true;
        }
      });
      if (needsUpdate) {
        selectedTypeSubject.add(updatedType);
        fetchCardListViewService(updatedType, context);
        safeSetState?.call(() {});
      }
    }
  }

  Future<ApiCallResponse?> fetchAccessHistoryService(
      BuildContext context) async {
    try {
      safeSetState.call(() => loading = true);
      var response = await PhpGroup.getAccessCall.call(
        pageSize: pageSize.toString(),
        pageNumber: pageNumber.toString(),
        pesTipo: personType != 'E' && personType != 'O' ? 'T' : personType,
      );

      final List<dynamic> accessHistory = response.jsonBody['acessos'] ?? [];

      List<dynamic> filteredAccess = accessHistory.where((item) {
        final personTypeMatches =
            personType == '.*' || item["PES_TIPO"].toString() == personType;
        return personTypeMatches;
      }).toList();

      if (filteredAccess != null && filteredAccess.isNotEmpty) {
        safeSetState.call(() {
          accessWrap.addAll(filteredAccess);
          hasData = true;
          loading = false;
        });
        return response;
      }
      _showNoMoreDataSnackbar(context);
      safeSetState.call(() {
        hasData = false;
        loading = false;
      });
      return null;
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed('processRequest', "", "Busca Acesso", e, s);
      safeSetState.call(() {
        hasData = false;
        loading = false;
      });
    }
    return null;
  }

  void _showNoMoreDataSnackbar(BuildContext context) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        content: Text(
          FFLocalizations.of(context).getVariableText(
              ptText: "Não há mais dados.", enText: "No more data."),
        ),
        duration: const Duration(seconds: 3),
        backgroundColor: FlutterFlowTheme.of(context).primary,
      ),
    );
  }

  void _loadMoreAccess(BuildContext context) {
    if (hasData == true) {
      pageNumber++;
      accessFuture = fetchAccessHistoryService(context);
    }
  }

  void fetchCardListViewService(
      Map<String, String> select, BuildContext context) {
    personType = select['personType']!;
    accessWrap = [];
    pageNumber = 1;
    accessFuture = fetchAccessHistoryService(context);
  }

  Future toggleOptionsAction(BuildContext context) async {
    await showModalBottomSheet(
      isScrollControlled: true,
      backgroundColor: Colors.transparent,
      useSafeArea: true,
      context: context,
      builder: (context) {
        return GestureDetector(
          onTap: () => unfocusNode.canRequestFocus
              ? FocusScope.of(context).requestFocus(unfocusNode)
              : FocusScope.of(context).unfocus(),
          child: Padding(
            padding: MediaQuery.viewInsetsOf(context),
            child: const OptModalWidget(),
          ),
        );
      },
    );
  }

  @override
  void dispose() {
    unfocusNode.dispose();

    clearAccessHistoryCache();
  }
}
